/*
 *	Overview: Allows one to emulate an 8-pin PIC12C67x/PIC12CE67x with 
 *  a standard ICD running a 28-pin PIC16F876.
 *
 *	Header file for the Microchip
 *	PIC 12C671 chip
 *	PIC 12C672 chip
 *	PIC 12CE673 chip
 *	PIC 12CE674 chip
 *	Midrange Microcontrollers
 *
 *  Modified by Shane Tolmie 20th-1-00
 *
 *  Used with MPLAB ICD, converts the relevant pins on a PIC16F877 ICD to
 *  the equivalents on a PIC12C67x/PIC12CE67x.  Compile for PIC16F877
 *  then use hand made hardware adaptor to plug into the PIC12C672/PIC12CE674
 *  socket.
 *
 *  The PIC12CE674 is based on the same core as the 16F877, thus the
 *  memory mapped registers are virtually the same.  GPIO and TRIS on
 *  PIC12CE674 go to PORTB and TRISB, and the port pins are swapped
 *  around a bit to make up for the lack of one interrupt pin on a
 *  16F877.  Lack of int pin because RB7 and RB6 are used on ICD.
 *
 *  Usage:
 *  #define _12CE674 USED or #define _12C672 USED
 *  #include "xC67ICD.h" instead of <pic.h> in code running on ICD.
 *  Make up adaptor for 8-pin (PIC12C672) to 28-pin (PIC16F876), soldering:
 *
 *  Power(pin 1 on 8-pin PIC12C672)=Power(pin 20 on 28-pin 16F876), 
 *  Ground(pin 8)=Ground(pin 8),
 *  GP5(pin 2)=RB3(pin 24), GP4(pin 3)=RB2(pin 23), GP3(pin 4)=RB5(pin 26),
 *  GP2(pin 5)=RB0(pin 21), GP1(pin 6)=RB1(pin 22), GP0(pin 7)=RB4(pin 25)
 *
 *  Differences: There is no A/D support, and GP1 has no interrupt on change.
 *  Do not use commands such as TRIS=0B10101010 or GPIO=0B11001100 etc as
 *  the port pin order is completely changed (see above), instead set
 *  pins individually with GP4=1, etc.  Can use GPIO=0xFF to zero all ports,
 *  or TRIS=0xFF to make everything input.
 *
 *  To do: solder wires to PORTA also, giving full a/d capability.
 *
 *  TESTED: made up adaptor and tested port outputs 26-Jan-00, works OK.
 *
 */

#define CLRWDT()  asm("clrwdt");

static volatile unsigned char TMR0 @0x01;			//done
static volatile unsigned char PCL @0x02;				//done
static volatile unsigned char STATUS @0x03;		//done
static unsigned char FSR @0x04;									//done

//static volatile unsigned char	GPIO	@ 0x05;		//done
static volatile unsigned char PORTB @0x06;			//done, GPIO is equivalent to PORTB
static volatile unsigned char GPIO @0x06;			//done, added for GPIO=0 or 0xFF instruction, nothing else
								//	as ports are all out of order going from PORTB to GPIO

static unsigned char PCLATH @0x0A;						//done
static volatile unsigned char INTCON @0x0B;		//done
static volatile unsigned char PIR1 @0x0C;			//done
static volatile unsigned char ADRES @0x1E;			//done
static volatile unsigned char ADCON0 @0x1F;		//done

static unsigned char bank1 OPTION @0x81;			//done

//static volatile	unsigned char bank1	TRIS	@ 0x85;
static volatile unsigned char bank1 TRISB @0x86; //done, TRIS is equivalent to TRISB
static volatile unsigned char bank1 TRIS @0x86; //done, added for TRIS=0 or 0xFF instruction, nothing else

static volatile unsigned char bank1 PIE1 @0x8C; //done
static volatile unsigned char bank1 PCON @0x8E; //done

//static volatile unsigned char bank1	OSCCAL	@ 0x8F; //done - no equivalent
unsigned char osccal_variable;
#define OSCCAL osccal_variable //dummy variable osccal_variable

static volatile unsigned char bank1 ADCON1 @0x9F; //done

/*	STATUS bits	*/ //same as PIC16F877
static volatile bit RP0 @(unsigned)&STATUS*8+5;				//done
static volatile bit TO @(unsigned)&STATUS*8+4;				//done
static volatile bit PD @(unsigned)&STATUS*8+3;				//done
static volatile bit ZERO @(unsigned)&STATUS*8+2;		//done
static volatile bit DC @(unsigned)&STATUS*8+1;				//done
static volatile bit CARRY @(unsigned)&STATUS*8+0;		//done

/*	GPIO bits	*/ //no equivalent on PIC16F877
/*
#if defined(_12CE673) || defined(_12CE674)
static volatile bit	SCL	@ (unsigned)&GPIO*8+7; //done
static volatile bit	SDA	@ (unsigned)&GPIO*8+6; //done
#endif
*/

//GP5/OSC1/CLKIN - I/O
//static volatile bit	GP5	@ (unsigned)&GPIO*8+5; //done
static volatile bit GP5 @(unsigned)&PORTB*8+3; //done, equivalent to GP5=RB3

//GP4/OSC2/AN3/CLKOUT - I/O
//static volatile bit	GP4	@ (unsigned)&GPIO*8+4; //done
static volatile bit GP4 @(unsigned)&PORTB*8+2; //done, equivalent to GP4=RB2

//GP3/MCLR/VPP - INPUT ONLY - INT ON CHANGE
//static volatile bit	GP3	@ (unsigned)&GPIO*8+3; //done
static volatile bit GP3 @(unsigned)&PORTB*8+5; //done, equivalent to GP3=RB5

//GP2/T0CKI/AN2/INT - I/O - INT ON LEADING/FALLING EDGE DETECT
//static volatile bit	GP2	@ (unsigned)&GPIO*8+2; //done
static volatile bit GP2 @(unsigned)&PORTB*8+0; //done, equivalent to GP2=RB0

//GP1/AN1/VREF - I/O - INT ON CHANGE
//static volatile bit	GP1	@ (unsigned)&GPIO*8+1; //done
static volatile bit GP1 @(unsigned)&PORTB*8+1; //done, equivalent to GP1=RB1
//theres not enough free interrupt pins - so this one misses out

//GP0/AN0 - I/O - INT ON CHANGE
//static volatile bit	GP0	@ (unsigned)&GPIO*8+0; //done
static volatile bit GP0 @(unsigned)&PORTB*8+4; //done, equivalent to GP0=RB4

/*	INTCON bits	*/
static volatile bit GIE @(unsigned)&INTCON*8+7; //done
static volatile bit PEIE @(unsigned)&INTCON*8+6; //done
static volatile bit T0IE @(unsigned)&INTCON*8+5; //done
static volatile bit INTE @(unsigned)&INTCON*8+4; //done
static volatile bit GPIE @(unsigned)&INTCON*8+3; //done
static volatile bit T0IF @(unsigned)&INTCON*8+2; //done
static volatile bit INTF @(unsigned)&INTCON*8+1; //done
static volatile bit GPIF @(unsigned)&INTCON*8+0; //done

/*	PIR1 bits */ //not used
//static volatile bit	ADIF	@ (unsigned)&PIR1*8+6;

/*	ADCON0 bits	*/

static volatile bit ADCS1 @(unsigned)&ADCON0*8+7;
static volatile bit ADCS0 @(unsigned)&ADCON0*8+6;
static volatile bit CHS1 @(unsigned)&ADCON0*8+4;
static volatile bit CHS0 @(unsigned)&ADCON0*8+3;
static volatile bit GODONE @(unsigned)&ADCON0*8+2;
static volatile bit ADON @(unsigned)&ADCON0*8+0;


/*	OPTION bits	*/
static bank1 bit GPPU @(unsigned)&OPTION*8+7; //done
static bank1 bit INTEDG @(unsigned)&OPTION*8+6; //done
static bank1 bit T0CS @(unsigned)&OPTION*8+5; //done
static bank1 bit T0SE @(unsigned)&OPTION*8+4; //done
static bank1 bit PSA @(unsigned)&OPTION*8+3; //done
static bank1 bit PS2 @(unsigned)&OPTION*8+2; //done
static bank1 bit PS1 @(unsigned)&OPTION*8+1; //done
static bank1 bit PS0 @(unsigned)&OPTION*8+0; //done


/* TRIS bits - see GPIO bits */
//static volatile bank1 bit        TRIS5  @ (unsigned)&TRIS*8+5; //done
static volatile bank1 bit TRIS5 @(unsigned)&TRISB*8+3; //done, TRIS5=TRISB3

//static volatile bank1 bit        TRIS4  @ (unsigned)&TRIS*8+4; //done
static volatile bank1 bit TRIS4 @(unsigned)&TRISB*8+2; //done, TRIS4=TRISB2

//static volatile bank1 bit        TRIS3  @ (unsigned)&TRIS*8+3; //done
static volatile bank1 bit TRIS3 @(unsigned)&TRISB*8+5; //done, TRIS3=TRISB5

//static volatile bank1 bit        TRIS2  @ (unsigned)&TRIS*8+2; //done
static volatile bank1 bit TRIS2 @(unsigned)&TRISB*8+0; //done, TRIS2=TRISB0

//static volatile bank1 bit        TRIS1  @ (unsigned)&TRIS*8+1; //done
static volatile bank1 bit TRIS1 @(unsigned)&TRISB*8+1; //done, TRIS1=TRISB1

//static volatile bank1 bit        TRIS0  @ (unsigned)&TRIS*8+0; //done
static volatile bank1 bit TRIS0 @(unsigned)&TRISB*8+4; //done, TRIS0=TRISB4


/*	PIE1 bits	*/
static volatile bank1 bit ADIE @(unsigned)&PIE1*8+6;		//done

/*	PCON bits */
static volatile bit POR @(unsigned)&PCON*8+1;

/*	OSCCAL bits */
#if defined(_12C671) || defined(_12C672)

//static volatile bank1 bit	CAL3	@ (unsigned)&OSCCAL*8+7;
#define CAL3 (osccal_variable>>7)&0x1;

//static volatile bank1 bit	CAL2	@ (unsigned)&OSCCAL*8+6;
#define CAL2 (osccal_variable>>6)&0x1;

//static volatile bank1 bit	CAL1	@ (unsigned)&OSCCAL*8+5;
#define CAL1 (osccal_variable>>5)&0x1;

//static volatile bank1 bit	CAL0	@ (unsigned)&OSCCAL*8+4;
#define CAL0 (osccal_variable>>4)&0x1;

//static volatile bank1 bit	CALFST	@ (unsigned)&OSCCAL*8+3;
#define CALFST (osccal_variable>>3)&0x1;

//static volatile bank1 bit	CALSLW	@ (unsigned)&OSCCAL*8+2;
#define CALSLW (osccal_variable>>2)&0x1;

#elif defined(_12CE673) || defined(_12CE674)

//static volatile bank1 bit	CAL5	@ (unsigned)&OSCCAL*8+7;
#define CAL5 (osccal_variable>>7)&0x1;

//static volatile bank1 bit	CAL4	@ (unsigned)&OSCCAL*8+6;
#define CAL4 (osccal_variable>>6)&0x1;

//static volatile bank1 bit	CAL3	@ (unsigned)&OSCCAL*8+5;
#define CAL3 (osccal_variable>>5)&0x1;

//static volatile bank1 bit	CAL2	@ (unsigned)&OSCCAL*8+4;
#define CAL2 (osccal_variable>>4)&0x1;

//static volatile bank1 bit	CAL1	@ (unsigned)&OSCCAL*8+3;
#define CAL1 (osccal_variable>>3)&0x1;

//static volatile bank1 bit	CAL0	@ (unsigned)&OSCCAL*8+2;
#define CAL0 (osccal_variable>>2)&0x1;

#endif

/*	ADCON1 bits */

/* if we're compiling for target micro, PIC12CE674, we want these bits to
function as per normal.  However, if we're compiling for the MPLAB-ICD using the
PIC16F876, its different.

There is no single pin on the MPLAB-ICD for 16F876 which is both analogue a/d,
and has interrupts.  Therefore, each pin on PIC12CE674 goes into 2 pins on the
MPLAB-ICD.  For example, GP2/AN2/INT on the PIC12CE674	goes into 2 pins on the
16F876, RB0 and RA2.  RA2 is always set to analogue input, and RB0 can be input
or output, and handles the interrupts.

The upshot of this is that if the MPLAB-ICD 16F876 is used, PORTA should always
only be set to analogue and input, and ADCON1 should never change .  Therefore,
if the MPLAB-ICD 16F876 is being used, redirect the PCFGx statements into the
void, by making them reference the unused top 3 bits of ADCON1*/

//static volatile bank1 bit	PCFG2	@ (unsigned)&ADCON1*8+5;

//compile for targe PIC12CE674
#if (ICD_EMULATION==0)
static volatile bank1 bit PCFG2 @(unsigned)&ADCON1*8+2;
static volatile bank1 bit PCFG1 @(unsigned)&ADCON1*8+1;
static volatile bank1 bit PCFG0 @(unsigned)&ADCON1*8+0;
#endif

//ensure that ADCON1 is always 0, analogue, when compiling for target MPLAB-ICD 16F876
#if (ICD_EMULATION==1)
unsigned char temp_adcon1;
#define PCFG2	temp_adcon1		//this results in temp_adcon1=N, which is effectively nothing
#define PCFG1	temp_adcon1
#define PCFG0	temp_adcon1
#endif


/*	Calibration constant in ROM */
#if defined(_12C671) || defined(_12CE673)
#define	_READ_OSCCAL_DATA()	(*(unsigned char(*)())0x3FF)()
#endif

#if defined(_12C672) || defined(_12CE674)
#define	_READ_OSCCAL_DATA()	(*(unsigned char(*)())0x7FF)()
#endif

#define	__CONFIG(x)	asm("\tpsect config,class=CODE,delta=2");\
			asm("\tglobal\tconfig_word"); \
			asm("config_word"); \
			asm("\tdw "___mkstr(x))

#define CONFIG_ADDR	0x2007
#define FOSC0		0x01
#define FOSC1		0x02
#define WDTE		0x04
#define PWRTE		0x08
#define CP0		0x1010
#define CP1		0x2020
#define BODEN		0x40
#define LVP		0x80
#define CPD		0x100
#define WRT		0x200
#define BKBUG		0x800

#define UNPROTECT	(CP0 | CP1)
#define PROTECT		0x0000

#if !defined(_16F870) && !defined(_16F871)
#define	PROTECT_TOP	CP1	/* protects top 256 bytes, 1F00h to 1FFFh	*/
#define PROTECT50	CP0	/* protects upper half of address space		*/
/* 1000h to 1FFFh (pic16f887/876)		*/
/* 0800h to 0FFFh (pic16f874/873)		*/
#endif


//*****
//tmr1 is only available to 16F876 when emulating 12C67X, its useful for timing
//how long code takes
static volatile unsigned char	TMR1L	@ 0x0E;
static volatile unsigned char	TMR1H	@ 0x0F;
static volatile unsigned char	T1CON	@ 0x10;

/*	PIR1 bits	*/
static volatile bit	TMR1IF	@ (unsigned)&PIR1*8+0;

/*	T1CON bits	*/
static volatile bit	T1CKPS1	@ (unsigned)&T1CON*8+5;
static volatile bit	T1CKPS0	@ (unsigned)&T1CON*8+4;
static volatile bit	T1OSCEN	@ (unsigned)&T1CON*8+3;
static volatile bit	T1SYNC 	@ (unsigned)&T1CON*8+2;
static volatile bit	TMR1CS 	@ (unsigned)&T1CON*8+1;
static volatile bit	TMR1ON 	@ (unsigned)&T1CON*8+0;

/*	PIE1 bits	*/
static volatile bank1 bit	TMR1IE	@ (unsigned)&PIE1*8+0;

